//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * Lampマテリアルユーティリティヘッダ
 * @author Junpee
 */

#ifndef LAMP_MATERIAL_UTILITY_H_
#define LAMP_MATERIAL_UTILITY_H_

namespace LampForMaya{

//------------------------------------------------------------------------------
/**
 * Lampマテリアルユーティリティ
 */
class LampMaterialUtility{
public:
	//--------------------------------------------------------------------------
	// テクスチャ
	//--------------------------------------------------------------------------
	/**
	 * テクスチャの取得
	 * @param node アトリビュートを持つノード
	 * @param attributeName テクスチャを持つアトリビュート名
	 * @return テクスチャ
	 */
	static MObject getTexture(
		const MObject& node, const String& attributeName);

	/**
	 * テクスチャ名の取得
	 * @param node アトリビュートを持つノード
	 * @param attributeName テクスチャを持つアトリビュート名
	 * @return テクスチャ名
	 */
	static String getTextureName(
		const MObject& node, const String& attributeName);

	//--------------------------------------------------------------------------
	// アトリビュート初期化
	//--------------------------------------------------------------------------
	/**
	 * ベーステクスチャアトリビュートの追加
	 */
	static void addBaseTextureAttribute();

	/**
	 * 光沢テクスチャアトリビュートの追加
	 */
	static void addGlossTextureAttribute();

	/**
	 * ライトテクスチャアトリビュートの追加
	 */
	static void addLightTextureAttribute();

	/**
	 * 汚れテクスチャアトリビュートの追加
	 */
	static void addStainTextureAttribute();

	/**
	 * ディフューズアトリビュートの追加
	 */
	static void addDiffuseAttribute();

	/**
	 * スペキュラアトリビュートの追加
	 */
	static void addSpecularAttribute();

	/**
	 * アンビエントアトリビュートの追加
	 */
	static void addAmbientAttribute();

	/**
	 * エミッシブアトリビュートの追加
	 */
	static void addEmissiveAttribute();

private:
	/**
	 * コンストラクタ
	 */
	LampMaterialUtility();

};

//------------------------------------------------------------------------------
} // End of namespace LampForMaya
#endif // End of LAMP_MATERIAL_UTILITY_H_
//------------------------------------------------------------------------------

